#!/bin/sh

# shellcheck disable=SC2039,SC2154

# This helper script will handle deploying a cloudfront CDN
set -e

source /scripts/utils.sh
source /scripts/terraform.sh

check_access "${appsNamespace}" configmaps
check_access "${infraNamespace}" configmaps

tfAdditionalVarsFile=${tfRoot}/additional_settings.auto.tfvars

terraform_extract_map_from_env_to_tf_vars -s "s3-cache-" -v "cache_bucket_arns" -f "${tfAdditionalVarsFile}"
terraform_extract_map_from_env_to_tf_vars -s "s3-domain-" -v "cache_bucket_regional_domains" -f "${tfAdditionalVarsFile}"
terraform_extract_map_from_env_to_tf_vars -s "lambda-arn-" -v "cache_lambda_arns" -f "${tfAdditionalVarsFile}"
terraform_extract_set_from_env_to_tf_vars -s "public-key-" -v "public_key_ids" -f "${tfAdditionalVarsFile}"

# Set Default s3 Cache Bucket Arn
echo "Found ARN for s3 default: \"${DEFAULT_CACHE_ARN}\""
echo "default_bucket_arn=\"${DEFAULT_CACHE_ARN}\"" >> "${tfAdditionalVarsFile}"

# Set Default s3 Cache Bucket Domain Name
echo "Found domain name for s3 default: \"${DEFAULT_CACHE_DOMAIN}\""
echo "default_bucket_domain_name=\"${DEFAULT_CACHE_DOMAIN}\"" >> "${tfAdditionalVarsFile}"

# Set Lambda ARN for Default Cache Lambda@Edge
if [ -z "${DEFAULT_CACHE_LAMBDA_ARN}" ] ; then
  echo "Found ARN for Lambda@Edge default: \"${DEFAULT_CACHE_LAMBDA_ARN}\""
  echo "default_cache_lambda_arn=\"${DEFAULT_CACHE_LAMBDA_ARN}\"" >> "${tfAdditionalVarsFile}"
fi

# Set Logging s3 Bucket Domain Name
echo "Found domain name for s3 logging: \"${LOGGING_DOMAIN}\""
echo "logging_bucket_domain_name=\"${LOGGING_DOMAIN}\"" >> "${tfAdditionalVarsFile}"

# Show additional .auto.tfvars file
print_header "Show additional_settings.auto.tfvars"
cat "$tfAdditionalVarsFile"

# Terraform Plan & Apply
terraform_init
terraform_plan
terraform_apply
terraform_output

## == Create Trusted Signer Secret ==
if printenv | grep "public-key-" ; then
  secretName=${namingPrefix}-signer-certificate
  print_header "Creating secret ${secretName} in namespace ${appsNamespace}"

  # Sort key-pair secrets by name in order of timestamp, parse most recent key-pair name
  most_recent_cert=$(kubectl get secrets -n "${appsNamespace}" --no-headers --sort-by=metadata.creationTimestamp -o custom-columns=":metadata.name" | grep key-pair | tail -1)
  echo "${most_recent_cert}"

  # Get most recent signer cert, extract key pair, trim quotes, decode, and export result to json
  encoded_string=$(kubectl get secrets -n "${appsNamespace}" "${most_recent_cert}" -o json | jq .data.keypair)
  temp="${encoded_string%\"}"
  encoded_string="${temp#\"}"
  echo "${encoded_string}" | base64 --decode | jq . >>signer_cert.json

  # Creates a new secret called '${namingPrefix}-signer-certificate' that contains the most recently created key pair
  kubectl --namespace "${appsNamespace}" create secret generic "${secretName}" \
    --save-config --dry-run=client \
    --from-file=signer_cert=./signer_cert.json \
    -o yaml | kubectl apply -f -
fi

## == Output values to ConfigMap ==
print_header "Dump output to ConfigMap"
arnConfigMapName=${namingPrefix}-arn
terraform_get arn arnValue
terraform_get id idValue
terraform_get domain_name domainNameValue
terraform_get key_group_id keyGroupIdValue

# Parse Path Patterns
terraform_get path_patterns pathPatternsOutput
pathPatternsValue=$(echo "${pathPatternsOutput}" | sed 's/ = /:/g;s/\"/\\"/g')
echo ${pathPatternsValue} > paths.txt
pathPatternsValue=$(cat paths.txt)

cat >/tmp/config.yaml <<EOF
apiVersion: v1
kind: ConfigMap
metadata:
  name: ${arnConfigMapName}
data:
  ARN: ${arnValue}
  ID: ${idValue}
  KEY_GROUP_ID: ${keyGroupIdValue}
  DOMAIN: ${domainNameValue}
  PATH_PATTERNS: "${pathPatternsValue}"
EOF

run_cmd cat "/tmp/config.yaml"
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${appsNamespace} apply -f /tmp/config.yaml
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${infraNamespace} apply -f /tmp/config.yaml

echo "Config map ${arnConfigMapName}"
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${appsNamespace} get cm ${arnConfigMapName} -o yaml

cprint "$0 done"
